@@ -31,9 +31,11 @@ from easytext.utils.json_util import json2str
 from easytext.utils.nn import cuda_util
 from easytext.trainer.metric_tracker import MetricTracker
 from easytext.trainer.grad_rescaled import GradRescaled
+from easytext.trainer import Record
+from easytext.trainer.trainer_callback import TrainerCallback
 
 
-class Trainer:
+class Trainer(TrainerCallback):
     """
     训练器
     """
@@ -51,7 +53,8 @@ class Trainer:
                  grad_scaled: GradRescaled = None,
                  patient: int = None,
                  num_check_point_keep: int = None,
-                 devices: Union[str, List[str]] = None):
+                 devices: Union[str, List[str]] = None,
+                 trainer_callback: TrainerCallback = None):
         """
         训练器初始化
         :param num_epoch: 训练的 epoch 数量
@@ -104,6 +107,7 @@ class Trainer:
         self._num_check_point_keep = num_check_point_keep
         self._num_epoch = num_epoch
         self._current_epoch: int = None
+        self._trainer_callback = trainer_callback
 
     @property
     def model(self):
@@ -321,7 +325,7 @@ class Trainer:
                              f"batch metrics: {json2str(batch_metrics)}, "
                              f"target metric: {json2str(target_metric)}")
 
-        # total_loss = total_loss / total_num 这是合理的 loss, 因为所有的 total_num 是一样的所以，没有一般要再除以一次了
+        # total_loss = total_loss / total_num 这是合理的 loss, 因为所有的 total_num 是一样的所以，没有必要再除以一次了
         return total_loss
 
     def recovery_train(self,
@@ -388,21 +392,38 @@ class Trainer:
 
         for epoch in range(start_epoch, self._num_epoch + 1):
 
+            record = Record()
+            record.epoch = epoch
+
             self._current_epoch = epoch
 
             logging.info(f"Start train epoch: {self._current_epoch}")
+            self.on_train_epoch_start(trainer=self, record=record)
 
             train_loss = self._train_or_evaluate(phrase=Trainer._TRAIN,
                                                  data_loader=train_data_loader)
+            record.epoch_train_loss = train_loss
 
             # 输出metrics
             train_metric_dict, train_target_metric = self._metrics.metric
+
+            record.train_metric = train_metric_dict
+            record.train_target_metric = train_target_metric
+
             logging.info(
                 f"Train epoch: {epoch}, loss: {train_loss}, target metric: {train_target_metric.name}:{train_target_metric.value} "
                 f"metrics: {json2str(train_metric_dict)}")
 
-            evaluate_loss = self.evaluate(validation_data_loader=validation_data_loader)
+            self.on_train_epoch_stop(trainer=self, record=record)
+
+            self.on_evaluate_validation_epoch_start(trainer=self, record=record)
+
+            validation_loss = self.evaluate(validation_data_loader=validation_data_loader)
+            record.epoch_validation_loss = validation_loss
+
             validation_metric_dict, validation_target_metric = self._metrics.metric
+            record.validation_metric = validation_metric_dict
+            record.validation_target_metric = validation_target_metric
 
             self._metric_tracker.add_metric(epoch=epoch,
                                             train_metric=train_metric_dict,
@@ -410,10 +431,12 @@ class Trainer:
                                             validation_metric=validation_metric_dict,
                                             validation_model_target_metric=validation_target_metric)
             logging.info(
-                f"Evaluate Valid epoch: {epoch}, loss: {evaluate_loss}, "
+                f"Evaluate Valid epoch: {epoch}, loss: {validation_loss}, "
                 f"target metric: {validation_target_metric.name}:{validation_target_metric.value} "
                 f"metrics: {json2str(validation_metric_dict)}")
 
+            self.on_evaluate_validation_epoch_stop(trainer=self, record=record)
+
             # 设置 lr scheduler
             # 注意这样设置会有点问题，对于某些 scheduler step 需要参数, 例如: ReduceLROnPlateau
             # 这种情况，暂时不处理, 如果处理需要重新重构，或者简单实用 isinstance ReduceLROnPlateau 来处理
@@ -421,7 +444,7 @@ class Trainer:
 
             if self._lr_scheduler is not None:
                 if isinstance(self._lr_scheduler, ReduceLROnPlateau):
-                    self._lr_scheduler.step(metrics=evaluate_loss, epoch=epoch)
+                    self._lr_scheduler.step(metrics=validation_loss, epoch=epoch)
                 else:
                     self._lr_scheduler.step(epoch=epoch)
 
@@ -430,3 +453,30 @@ class Trainer:
             if self._metric_tracker.early_stopping(epoch):
                 logging.info(f"Epoch: {epoch}, early stopping!")
                 break
+
+        self.on_training_complete(trainer=self, record=record)
+
+    def on_train_epoch_start(self, trainer: "Trainer", record: Record) -> None:
+        logging.info(f"on_train_epoch_start: {record.epoch}")
+        self._trainer_callback.on_train_epoch_start(trainer=trainer,
+                                                    record=record)
+
+    def on_train_epoch_stop(self, trainer: "Trainer", record: Record) -> None:
+        logging.info(f"on_train_epoch_stop: {record.epoch}")
+        self._trainer_callback.on_train_epoch_stop(trainer=trainer,
+                                                   record=record)
+
+    def on_evaluate_validation_epoch_start(self, trainer: "Trainer", record: Record) -> None:
+        logging.info(f"on_evaluate_epoch_start: {record.epoch}")
+        self._trainer_callback.on_evaluate_validation_epoch_start(trainer=trainer,
+                                                                  record=record)
+
+    def on_evaluate_validation_epoch_stop(self, trainer: "Trainer", record: Record) -> None:
+        logging.info(f"on_evaluate_epoch_stop: {record.epoch}")
+        self._trainer_callback.on_evaluate_validation_epoch_stop(trainer=trainer,
+                                                                 record=record)
+
+    def on_training_complete(self, trainer: "Trainer", record: Record) -> None:
+        logging.info(f"on_training_complete: {record.epoch}")
+        self._trainer_callback.on_training_complete(trainer=trainer,
+                                                    record=record)
